﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Text;
using System.Text.RegularExpressions;

namespace ConsoleDrapperDemo
{
    public partial class DbHelp : IDisposable
    {
        enum DBType
        {
            SqlServer2000,
            SqlServer,
            Oracle,
            SQLite
        }

        #region 成员定义
        public IDbConnection Dapper
        {
            get
            {
                if (conn != null) return conn;
                conn = _provider.CreateConnection();
                conn.ConnectionString = connectionString;
                conn.Open();
                return conn;
            }
        }

        private DbProviderFactory _provider;//数据库Provider
        private DBType _dbType;//数据库类型
        private char _paramPrefix = '@';//参数符号

        private DbConnection conn;//连接对象
        private string connectionString;//连接字符串
        private DbTransaction tran;//事务对象

        private IList parameterList = new List<DbParameter>();//过程参数列表
        private bool hasOutput = false;//是否包含输出参数
        private Dictionary<string, object> dicPara = new Dictionary<string, object>();//输出参数列表
        #endregion

        #region 构造方法，实例化连接字符串

        /// <summary>
        /// 读取WebConfig链接字符串
        /// </summary>
        /// <param name="connectionName">ConnectionString配置名</param>
        public DbHelp(string connectionName = "local")
        {
            //默认使用ConnectionString第一项
            var config = ConfigurationManager.ConnectionStrings[connectionName];
            _provider = DbProviderFactories.GetFactory(config.ProviderName);
            connectionString = config.ConnectionString;
            CommonConstruct(config.ProviderName);
        }

        /// <summary>
        /// 有参构造，实例化连接字符串
        /// </summary>
        /// <param name="provider">DbProvider</param>
        /// <param name="connectionString">连接字符串</param>
        public DbHelp(DbProviderFactory provider, string connectionString)
        {
            this._provider = provider;
            this.connectionString = connectionString;
            CommonConstruct(provider.GetType().Name);
        }

        private void CommonConstruct(string _dbtype = "")
        {
            // Try using type name first (more reliable)
            if (_dbtype.StartsWith("Oracle")) _dbType = DBType.Oracle;
            else if (_dbtype.StartsWith("SQLite")) _dbType = DBType.SQLite;
            else if (_dbtype.StartsWith("System.Data.SqlClient")) _dbType = DBType.SqlServer;
            // else try with provider name
            else if (_dbtype.IndexOf("Oracle", StringComparison.InvariantCultureIgnoreCase) >= 0) _dbType = DBType.Oracle;
            else if (_dbtype.IndexOf("SQLite", StringComparison.InvariantCultureIgnoreCase) >= 0) _dbType = DBType.SQLite;

            if (_dbType == DBType.Oracle)
                _paramPrefix = ':';
            else
                _paramPrefix = '@';
        }
        #endregion

        #region 实现接口IDisposable
        /// <释放资源接口>
        /// 实现接口IDisposable
        /// </释放资源接口>
        public void Dispose()
        {
            if (conn != null)
            {
                if (conn.State == ConnectionState.Open)//判断数据库连接池是否打开
                {
                    conn.Close();
                }

                if (parameterList.Count > 0)//判断参数列表是否清空
                {
                    parameterList.Clear();
                }
                conn.Dispose();//释放连接池资源
                GC.SuppressFinalize(this);//垃圾回收
            }
        }
        #endregion

        #region 执行SQL或存储过程 并返回影响的行数
        /// <summary>
        /// 执行SQL，并返回影响的行数
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <returns></returns>
        public int ExecuteNonQuery(string sql)
        {
            using (var cmd = CreateCommand(sql))
            {
                return ExecuteNonQuery(cmd);
            }
        }

        /// <summary>
        /// 执行存储过程，并返回影响的行数
        /// </summary>
        /// <param name="storeProcedureName">存储过程名</param>
        /// <returns></returns>
        public int ExecuteProceudre(string storeProcedureName)
        {
            using (var cmd = CreateCommand(storeProcedureName, CommandType.StoredProcedure))
            {
                return ExecuteNonQuery(cmd);
            }
        }
        #endregion

        #region 执行SQL或者存储过程，并返回DataTable
        /// <summary>
        /// 执行SQL语句并返回DataTable
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <returns></returns>
        public DataTable ExecuteSql(string sql)
        {
            using (var cmd = CreateCommand(sql))
            {
                return Execute(cmd);
            }
        }

        /// <summary>
        /// 执行存储过程并返回DataTable
        /// </summary>
        /// <param name="storeProcedureName">存储过程名</param>
        /// <returns></returns>
        public DataTable ExecuteProc(string storeProcedureName)
        {
            using (var cmd = CreateCommand(storeProcedureName, CommandType.StoredProcedure))
            {
                return Execute(cmd);
            }
        }
        #endregion

        #region 执行SQL或存储过程并返回DbDataReader
        /// <summary>
        /// 执行SQL语句并返回DbDataReader
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <returns>返回DbDataReader</returns>
        public DbDataReader ExecuteReader(string sql)
        {
            using (var cmd = CreateCommand(sql))
            {
                return ExecuteReader(cmd);
            }
        }

        /// <summary>
        /// 执行存储过程并返回DbDataReader
        /// </summary>
        /// <param name="storeProcedureName">存储过程名</param>
        /// <returns>返回DbDataReader</returns>
        public DbDataReader ExecuteProcReader(string storeProcedureName)
        {
            using (var cmd = CreateCommand(storeProcedureName, CommandType.StoredProcedure))
            {
                return ExecuteReader(cmd);
            }
        }
        #endregion

        #region 执行统计
        /// <summary>
        /// 执行SQL语句 返回首行首列的值,一般用于统计
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <returns>查询结果首行首列的值转换为整形，转换失败则返回-1</returns>
        public int Count(string sql)
        {
            using (var cmd = CreateCommand(sql))
            {
                return ExecuteScalar(cmd);
            }
        }
        #endregion

        #region 测试连接是否成功
        /// <summary>
        /// 测试连接是否成功
        /// </summary>
        /// <returns></returns>
        public bool HasConnection
        {
            get
            {
                try
                {
                    conn = new SqlConnection(connectionString);
                    conn.Open();
                    return true;
                }
                catch
                {
                    return false;
                }
                finally
                {
                    conn.Close();
                }
            }
        }
        #endregion

        #region 索引器访问
        public object this[string name]
        {
            set
            {
                this[name, DbType.Object, ParameterDirection.Input] = value;
            }
            get
            {
                object obj;
                if (dicPara.TryGetValue(name, out obj))
                {
                    return obj;
                }
                return null;
            }
        }

        public object this[string name, DbType dbtype]
        {
            set
            {
                this[name, dbtype, ParameterDirection.Input] = value;
            }
        }

        public object this[string name, DbType dbType, ParameterDirection direction]
        {
            set
            {
                if (name[0] != _paramPrefix) name = _paramPrefix + name;

                var para = _provider.CreateParameter();
                if (dbType != DbType.Object)
                    para.DbType = dbType;
                para.ParameterName = name;
                para.Value = value == null ? DBNull.Value : value;
                parameterList.Add(para);
            }
        }
        #endregion

        #region 命令相关处理
        /// <summary>
        /// 创建DbCommand
        /// </summary>
        /// <param name="cmdText">命名文本</param>
        /// <param name="cmdType">命名类型</param>
        /// <returns></returns>
        private DbCommand CreateCommand(string cmdText, CommandType cmdType = CommandType.Text)
        {
            //创建数据库连接对象
            if (conn == null || conn.State != ConnectionState.Open)
            {
                conn = _provider.CreateConnection();
                conn.ConnectionString = connectionString;
                conn.Open();//打开数据库连接池
            }

            //创建Command命令
            var cmd = conn.CreateCommand();
            cmd.Connection = conn;
            cmd.CommandType = cmdType;
            if (!string.IsNullOrEmpty(cmdText))
                cmd.CommandText = cmdText;
            if (tran != null) cmd.Transaction = tran;
            cmd.CommandTimeout = 0;

            //加载过程参数
            LoadParamter(cmd);
            return cmd;
        }

        /// <summary>
        /// 创建过程参数
        /// </summary>
        /// <param name="name">参数名</param>
        /// <param name="value">参数值</param>
        /// <param name="t">参数值类型</param>
        /// <param name="pDirection">参数类型</param>
        /// <returns></returns>
        private DbParameter CreateParameter(string name, object value, DbType t = DbType.Object, ParameterDirection pDirection = ParameterDirection.Input)
        {
            var para = _provider.CreateParameter();
            if (t != DbType.Object) para.DbType = t;
            para.Direction = pDirection;
            if (name[0] == _paramPrefix)
            {
                para.ParameterName = name;
            }
            else
            {
                para.ParameterName = _paramPrefix + name;
            }
            para.Value = value;
            return para;
        }

        /// <summary>
        /// 执行Command 并返回影响的行数
        /// </summary>
        /// <param name="cmd">命令</param>
        /// <returns></returns>
        private int ExecuteNonQuery(DbCommand cmd)
        {
            try
            {
                return cmd.ExecuteNonQuery();
            }
            catch (Exception)
            {
                conn.Close();
                throw;
            }
            finally
            {
                if (tran == null) Dispose();
            }
        }

        /// <summary>
        /// 执行Command 并返回影响的行数
        /// </summary>
        /// <param name="cmd">命令</param>
        /// <returns></returns>
        private int ExecuteScalar(DbCommand cmd)
        {
            try
            {
                var ret = cmd.ExecuteScalar().ToString();
                int i;
                if (!int.TryParse(ret, out i))
                {
                    throw new Exception("can't parse it to int,the value is " + ret);
                }
                return i;
            }
            catch (Exception)
            {
                conn.Close();
                throw;
            }
            finally
            {
                if (tran == null) Dispose();
            }
        }

        /// <summary>
        /// 执行Command 并返回DataTable
        /// </summary>
        /// <param name="cmd">命令</param>
        /// <returns></returns>
        private DataTable Execute(DbCommand cmd)
        {
            try
            {
                using (var adapter = _provider.CreateDataAdapter())//创建适配器
                {
                    adapter.SelectCommand = cmd;
                    adapter.SelectCommand.CommandTimeout = 0;
                    var dt = new DataTable();
                    adapter.Fill(dt);
                    return dt;//返回结果集
                }
            }
            catch (Exception)
            {
                conn.Close();
                throw;
            }
            finally
            {
                if (tran == null) Dispose();
            }
        }

        /// <summary>
        /// 执行Command 并返回DbDataReader
        /// </summary>
        /// <param name="cmd">命令</param>
        /// <returns></returns>
        private DbDataReader ExecuteReader(DbCommand cmd)
        {
            try
            {
                return cmd.ExecuteReader(CommandBehavior.CloseConnection);
            }
            catch (Exception)
            {
                conn.Close();
                throw;
            }
            finally
            {
                if (tran == null) Dispose();
            }
        }

        /// <summary>
        /// 加载输出参数至字典中 仅当执行非查询时才调用
        /// </summary>
        /// <param name="Parameters"></param>
        private void InitDic(DbParameterCollection Parameters)
        {
            if (hasOutput)
            {
                dicPara.Clear();
                foreach (DbParameter Para in Parameters)
                {
                    if (Para.Direction != ParameterDirection.Input)
                    {
                        dicPara.Add(Para.ParameterName, Para.Value);
                    }
                }
                hasOutput = false;
            }
        }

        /// <summary>
        /// 加载过程参数输入至Commond中
        /// </summary>
        /// <param name="cmd"></param>
        private void LoadParamter(DbCommand cmd)
        {
            if (parameterList.Count != 0)
            {
                foreach (DbParameter Para in parameterList)
                {
                    if (!hasOutput && Para.Direction != ParameterDirection.Input)
                    {
                        hasOutput = true;
                    }
                    cmd.Parameters.Add(Para);
                }
                parameterList.Clear();
            }
        }

        /// <summary>
        /// 将参数化Sql转换成纯Sql，便于调试
        /// </summary>
        /// <param name="strSql">SQL语句</param>
        /// <returns></returns>
        private string getSqlOnly(DbCommand cmd)
        {
            var sql = cmd.CommandText;
            foreach (DbParameter para in cmd.Parameters)
            {
                if (para.DbType == DbType.Int16 || para.DbType == DbType.Int32 || para.DbType == DbType.Int64 || para.DbType == DbType.UInt16 || para.DbType == DbType.UInt32 || para.DbType == DbType.UInt64 || para.DbType == DbType.Decimal || para.DbType == DbType.Double || para.DbType == DbType.Single)
                {
                    sql = sql.Replace(para.ParameterName, para.Value.ToString());
                }
                else if (_paramPrefix == '@' || para.DbType == DbType.AnsiString || para.DbType == DbType.String || para.DbType == DbType.StringFixedLength)
                {
                    sql = sql.Replace(para.ParameterName, "'" + para.Value.ToString() + "'");
                }
                else if (_paramPrefix == ':' || para.DbType == DbType.DateTime || para.DbType == DbType.DateTime2 || para.DbType == DbType.DateTimeOffset)
                {
                    //排除未知的时间类型
                    DateTime time;
                    if (DateTime.TryParse(para.Value.ToString(), out time))
                    {
                        sql = sql.Replace(para.ParameterName, "to_date('" + time.ToString() + "','yyyy-MM-dd hh24:mi:ss')");
                    }
                }
            }
            return sql;
        }
        #endregion

        #region 分页查询
        /// <summary>
        /// 对指定Sql语句查询的结果集进行分页
        /// </summary>
        /// <param name="sql">sql语句</param>
        /// <param name="start">结果集起始行号，不包括此行</param>
        /// <param name="limit">取出的行数</param>
        /// <returns></returns>
        public DataTable ExecuteSql(string sql, int start, int limit)
        {
            var sqls = CreatePageSql(sql, start, limit);
            using (var cmd = CreateCommand(sqls[1]))
            {
                CreatePageParameter(null, start, limit,cmd);
                return Execute(cmd);
            }
        }

        /// <summary>
        /// 对指定Sql语句查询的结果集进行分页
        /// </summary>
        /// <param name="sql">sql语句</param>
        /// <param name="start">结果集起始行号，不包括此行</param>
        /// <param name="limit">取出的行数</param>
        /// <returns></returns>
        public DbDataReader ExecuteReader(string sql, int start, int limit)
        {
            var sqls = CreatePageSql(sql, start, limit);
            using (var cmd = CreateCommand(sqls[1]))
            {
                CreatePageParameter(null, start, limit, cmd);
                return ExecuteReader(cmd);
            }
        }

        /// <summary>
        /// 对指定Sql语句查询的结果集进行分页
        /// </summary>
        /// <param name="sql">sql语句</param>
        /// <param name="start">结果集起始行号，不包括此行</param>
        /// <param name="limit">取出的行数</param>
        /// <param name="count">输出总行数</param>
        /// <returns></returns>
        public DataTable ExecuteSql(string sql, int start, int limit, out int count)
        {
            //查看是否已经使用事务,若没有使用事务，这里必须使用事务执行
            var alreadyUserTran = tran != null;
            if (!alreadyUserTran)
                BeginTransation();

            var sqls = CreatePageSql(sql, start, limit, true);
            using (var cmd = CreateCommand(sqls[0]))
            {
                count = ExecuteScalar(cmd);

                CreatePageParameter(null, start, limit, cmd);

                cmd.CommandText = sqls[1];
                var dt = Execute(cmd);

                //如果事先已开启事务，则不在此处提交事务，应由用户调用时手动提交，否则自动提交方法
                if (!alreadyUserTran)
                    tran.Commit();

                return dt;
            }
        }

        /// <summary>
        /// 对指定Sql语句查询的结果集进行分页
        /// </summary>
        /// <param name="sql">sql语句</param>
        /// <param name="start">结果集起始行号，不包括此行</param>
        /// <param name="limit">取出的行数</param>
        /// <param name="count">输出总行数</param>
        /// <returns></returns>
        public DbDataReader ExecuteReader(string sql, int start, int limit, out int count)
        {
            //查看是否已经使用事务,若没有使用事务，这里必须使用事务执行
            var alreadyUserTran = tran != null;
            if (!alreadyUserTran)
                BeginTransation();

            var sqls = CreatePageSql(sql,   start, limit, true);
            using (var cmd = CreateCommand(sqls[0]))
            {
                count = ExecuteScalar(cmd);

                CreatePageParameter(null, start, limit, cmd);

                cmd.CommandText = sqls[1];
                return ExecuteReader(cmd);
            }
        }
        #endregion

        #region 开启事务
        /// <summary>
        /// 开启事务
        /// </summary>
        /// <returns></returns>
        public void BeginTransation()
        {
            //创建数据库连接对象
            if (conn == null || conn.State != ConnectionState.Open)
            {
                conn = _provider.CreateConnection();
                conn.ConnectionString = connectionString;
                conn.Open();//打开数据库连接池
            }
            tran = conn.BeginTransaction();
        }

        /// <summary>
        /// 提交事务
        /// </summary>
        public void Commit()
        {
            if (tran != null)
            {
                tran.Commit();
                tran.Dispose();
                tran = null;
                Dispose();
            }
        }

        /// <summary>
        /// 回滚事务
        /// </summary>
        public void Rollback()
        {
            if (tran != null)
            {
                tran.Rollback();
                tran.Dispose();
                tran = null;
                Dispose();
            }
        }
        #endregion
    }
}
