﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using Entity = System.Collections.Generic.IDictionary<string, object>;
namespace ConsoleDrapperDemo
{
    #region Attribute定义
    // 标记该属性，则要求所有列都需要标记属性
    [AttributeUsage(AttributeTargets.Class)]
    public class ExplicitColumnsAttribute : Attribute
    {
    }
    // For non-explicit pocos, causes a property to be ignored
    [AttributeUsage(AttributeTargets.Property)]
    public class IgnoreAttribute : Attribute
    {
    }

    // For explicit pocos, marks property as a column and optionally supplies column name
    [AttributeUsage(AttributeTargets.Property)]
    public class ColumnAttribute : Attribute
    {
        public ColumnAttribute() { }
        public ColumnAttribute(string name) { Name = name; }
        public string Name { get; set; }
    }

    // For explicit pocos, marks property as a result column and optionally supplies column name
    [AttributeUsage(AttributeTargets.Property)]
    public class ResultColumnAttribute : ColumnAttribute
    {
        public ResultColumnAttribute() { }
        public ResultColumnAttribute(string name) : base(name) { }
    }

    // Specify the table name of a poco
    [AttributeUsage(AttributeTargets.Class)]
    public class TableNameAttribute : Attribute
    {
        public TableNameAttribute(string tableName)
        {
            Value = tableName;
        }
        public string Value { get; private set; }
    }

    // Specific the primary key of a poco class (and optional sequence name for Oracle)
    [AttributeUsage(AttributeTargets.Class)]
    public class PrimaryKeyAttribute : Attribute
    {
        public PrimaryKeyAttribute(string primaryKey)
        {
            Value = primaryKey;
            autoIncrement = true;
        }

        public string Value { get; private set; }
        public string sequenceName { get; set; }
        public bool autoIncrement { get; set; }
    }

    [AttributeUsage(AttributeTargets.Class)]
    public class OrderAttribute : Attribute
    {
        public OrderAttribute(string colName)
        {
            this.colName = colName;
        }

        public string colName { get; set; }
        public bool IsDesc { get; set; }
    }

    [AttributeUsage(AttributeTargets.Property)]
    public class AutoJoinAttribute : Attribute
    {
        public AutoJoinAttribute() { }
    }
    #endregion

    // Used by IMapper to override table bindings for an object
    public class TableInfo
    {
        public string TableName { get; set; }
        public string PrimaryKey { get; set; }
        public bool AutoIncrement { get; set; }
        public string SequenceName { get; set; }
    }

    public class PocoColumn
    {
        public string ColumnName;
        public PropertyInfo PropertyInfo;
        public bool ResultColumn;
        public virtual void SetValue(object target, object val) { PropertyInfo.SetValue(target, val, null); }
        public virtual object GetValue(object target) { return PropertyInfo.GetValue(target, null); }
        public virtual object ChangeType(object val) { return Convert.ChangeType(val, PropertyInfo.PropertyType); }
    }

    public class PocoData
    {
        public Dictionary<string, PocoColumn> Columns { get; private set; }
        public string[] QueryColumns { get; private set; }
        public string OrderBy { get; private set; }
        public TableInfo TableInfo { get; private set; }
        public Type type;

        static System.Threading.ReaderWriterLockSlim RWLock = new System.Threading.ReaderWriterLockSlim();
        static Dictionary<Type, PocoData> m_PocoDatas = new Dictionary<Type, PocoData>();


        public PocoData(Type t)
        {
            type = t;
            TableInfo = new TableInfo();

            //得到表名 未配置则使用类型名称
            var a = t.GetCustomAttributes(typeof(TableNameAttribute), true);
            TableInfo.TableName = a.Length == 0 ? t.Name : (a[0] as TableNameAttribute).Value;

            //得到主键 序列名 是否自增长等信息
            a = t.GetCustomAttributes(typeof(PrimaryKeyAttribute), true);
            if (a.Length == 0)
            {
                TableInfo.PrimaryKey = "ID";
                TableInfo.SequenceName = null;
                TableInfo.AutoIncrement = false;
            }
            else
            {
                var keyAttr = a[0] as PrimaryKeyAttribute;
                TableInfo.PrimaryKey = keyAttr.Value;
                TableInfo.SequenceName = keyAttr.sequenceName;
                TableInfo.AutoIncrement = keyAttr.autoIncrement;
            }

            //获取排序信息
            a = t.GetCustomAttributes(typeof(OrderAttribute), true);
            if (a.Length == 0)
            {
                OrderBy = string.Format("ORDER BY {0} DESC", TableInfo.PrimaryKey);
            }
            else
            {
                var orderAttr = a[0] as OrderAttribute;
                OrderBy = string.Format("ORDER BY {0} {1}", orderAttr.colName, orderAttr.IsDesc ? "DESC" : "ASC");
            }

            //获取是否标记所有列信息
            bool ExplicitColumns = t.GetCustomAttributes(typeof(ExplicitColumnsAttribute), true).Length > 0;

            Columns = new Dictionary<string, PocoColumn>(StringComparer.OrdinalIgnoreCase);
            foreach (var pi in t.GetProperties())
            {
                // Work out if properties is to be included
                var ColAttrs = pi.GetCustomAttributes(typeof(ColumnAttribute), true);
                if (ExplicitColumns)
                {
                    if (ColAttrs.Length == 0) continue;
                }
                else
                {
                    if (pi.GetCustomAttributes(typeof(IgnoreAttribute), true).Length != 0) continue;
                }

                var pc = new PocoColumn();
                pc.PropertyInfo = pi;

                // Work out the DB column name
                if (ColAttrs.Length > 0)
                {
                    var colattr = (ColumnAttribute)ColAttrs[0];
                    pc.ColumnName = colattr.Name;
                    if ((colattr as ResultColumnAttribute) != null)
                        pc.ResultColumn = true;
                }
                if (pc.ColumnName == null)
                {
                    pc.ColumnName = pi.Name;
                }

                // Store it
                Columns.Add(pc.ColumnName, pc);
            }

            // Build column list for automatic select
            QueryColumns = (from c in Columns where !c.Value.ResultColumn select c.Key).ToArray();
        }

        public static PocoData ForType(Type t)
        {
            //尝试从缓存取
            PocoData pd;
            try
            {
                RWLock.EnterReadLock();
                if (m_PocoDatas.TryGetValue(t, out pd))
                    return pd;
            }
            finally
            {
                RWLock.ExitReadLock();
            }

            //未取到则初始化并放入缓存
            RWLock.EnterWriteLock();
            try
            {
                // 再次检查
                if (m_PocoDatas.TryGetValue(t, out pd))
                    return pd;

                // Create it
                pd = new PocoData(t);
                m_PocoDatas.Add(t, pd);
            }
            finally
            {
                RWLock.ExitWriteLock();
            }

            return pd;
        }
    }

    public partial class DbHelp
    {
        public int Add(string tableName, string primaryKeyName, bool autoIncrement, object poco)
        {
            var t = PocoData.ForType(poco.GetType());

            //获取需要组织Insert的列
            string[] cols;
            //若实体包含自增长列，或者定义了序列名
            if (t.TableInfo.AutoIncrement || !string.IsNullOrEmpty(t.TableInfo.SequenceName))
            {
                cols = t.Columns.Where(c => c.Key != t.TableInfo.PrimaryKey).Select(c => c.Key).ToArray();
            }
            else //如果不包含，则所有列需要指定Insert
            {
                cols = t.Columns.Select(c => c.Key).ToArray();
            }

            var sql = string.Format("INSERT INTO {0} ({1}) VALUES ({2}{3})",
                tableName,
                string.Join(",", cols),
                _paramPrefix,
                string.Join("," + _paramPrefix, cols));
            return 0;
        }

        public int Add(object poco)
        {
            var t = PocoData.ForType(GetEntityType(poco));
            var sql = CreateInsert(t);
            return conn.Execute(sql, poco);
        }

        public int Update(object poco)
        {
            var t = PocoData.ForType(GetEntityType(poco));
            var sql = CreateUpdate(t);
            return conn.Execute(sql, poco);
        }

        public int Update<T>(IDictionary<string, object> updateValue, IDictionary<string, object> condition)
        {
            var t = PocoData.ForType(typeof(T));
            //创建Update语句
            var setCols = updateValue.Select(c => string.Format("{0}={1}{0}", c.Key, _paramPrefix)).ToArray();
            var update = string.Format("UPDATE {0} SET {1}", t.TableInfo.TableName, string.Join(",", setCols));
            var where = CreateWhere(t, condition);
            var sql = string.Format("{0} WHERE {1}", update, where);

            //合并字典
            foreach (var con in condition)
            {
                updateValue[con.Key] = con.Value;
            }
            return conn.Execute(sql, updateValue);
        }

        public int Remove(object poco)
        {
            var t = PocoData.ForType(GetEntityType(poco));

            var sql = string.Format("DELETE FROM {0} WHERE {1}={2}{1}",
                t.TableInfo.TableName,
                t.TableInfo.PrimaryKey,
                _paramPrefix);

            return conn.Execute(sql, poco);
        }

        public int Remove<T>(IDictionary<string, object> condition)
        {
            var t = PocoData.ForType(typeof(T));
            var where = CreateWhere(t, condition);

            var sql = string.Format("DELETE FROM {0} {1} {2}",
               t.TableInfo.TableName,
               string.IsNullOrEmpty(where) ? string.Empty : "WHERE",
               where);

            return conn.Execute(sql, condition);
        }

        public IEnumerable<T> Query<T>(IDictionary<string, object> condition)
        {
            var t = PocoData.ForType(typeof(T));
            var where = CreateWhere(t, condition);

            var sql = string.Format("SELECT {0} FROM {1} {2} {3} {4}",
               string.Join(",", t.QueryColumns),
               t.TableInfo.TableName,
               string.IsNullOrEmpty(where) ? string.Empty : "WHERE",
               where,
               t.OrderBy);

            return conn.Query<T>(sql, condition);
        }

        public IEnumerable<T> Query<T>(IDictionary<string, object> condition, int start, int limit)
        {
            //获取类型数据
            var t = PocoData.ForType(typeof(T));
            //获取查询条件
            var where = CreateWhere(t, condition);
            //获取查询语句
            var sql = string.Format("SELECT {0} FROM {1} {2} {3} {4}",
               string.Join(",", t.QueryColumns),
               t.TableInfo.TableName,
               string.IsNullOrEmpty(where) ? string.Empty : "WHERE",
               where,
               t.OrderBy);
            //获取分页SQL
            var sqls = CreatePageSql(sql, start, limit);
            //加载分页参数
            CreatePageParameter(condition, start, limit);
            return conn.Query<T>(sqls[1], condition);
        }

        public IEnumerable<T> Query<T>(IDictionary<string, object> condition, int start, int limit, out int count)
        {
            //获取类型数据
            var t = PocoData.ForType(typeof(T));
            //获取查询条件
            var where = CreateWhere(t, condition);
            //获取查询语句
            var sql = string.Format("SELECT {0} FROM {1} {2} {3} {4}",
               string.Join(",", t.QueryColumns),
               t.TableInfo.TableName,
               string.IsNullOrEmpty(where) ? string.Empty : "WHERE",
               where,
               t.OrderBy);
            //获取分页SQL
            //给Dictionary加参数前缀
            //condition = condition.ToDictionary(item => _paramPrefix + item.Key, item => item.Value);
            var sqls = CreatePageSql(sql, start, limit, true);
            //执行统计 这里不知道用Dapper该怎么更好的执行了
            var countResult = conn.Query(sqls[0], condition).SingleOrDefault();
            count = (int)countResult.TOTAL;
            //加载分页参数
            CreatePageParameter(condition, start, limit);
            return conn.Query<T>(sqls[1], condition);
        }

        #region 生成SQL语句
        private Type GetEntityType(object poco)
        {
            //查找真正的实体类型 参数可以是IEnumerable(批量插入),和object
            var multiExec = (object)poco as IEnumerable;
            if (multiExec != null && !(multiExec is string))
            {
                bool first = false;
                foreach (var obj in multiExec)
                {
                    first = true;
                    return obj.GetType();
                }
            }
            return poco.GetType();
        }

        private string CreateInsert(PocoData t)
        {
            //获取需要组织Insert的列
            string[] cols;

            //若实体包含自增长列，或者定义了序列名
            if (t.TableInfo.AutoIncrement || !string.IsNullOrEmpty(t.TableInfo.SequenceName))
            {
                cols = t.Columns.Where(c => c.Key != t.TableInfo.PrimaryKey).Select(c => c.Key).ToArray();
            }
            else //如果不包含，则所有列需要指定Insert
            {
                cols = t.Columns.Select(c => c.Key).ToArray();
            }

            //生成Insert语句
            var sql = string.Format("INSERT INTO {0} ({1}) VALUES ({2}{3})",
               t.TableInfo.TableName, //表名
               string.Join(",", cols),//显示指定需要插入的列名
               _paramPrefix,//过程参数缺少第一个参数符号,补充
               string.Join("," + _paramPrefix, cols));//指定需要插入的参数

            return sql;
        }

        private string CreateUpdate(PocoData t)
        {
            //获取需要组织Update的列，这里假设主键是非自增都有可能更新的
            string[] cols;

            //若实体包含自增长列，或者定义了序列名
            if (t.TableInfo.AutoIncrement || !string.IsNullOrEmpty(t.TableInfo.SequenceName))
            {
                cols = t.Columns.Where(c => c.Key != t.TableInfo.PrimaryKey).Select(c => string.Format("{0}={1}{0}", c.Key, _paramPrefix)).ToArray();
            }
            else //如果不包含，则所有列需要指定Insert
            {
                cols = t.Columns.Select(c => string.Format("{0}={1}{0}", c.Key, _paramPrefix)).ToArray();
            }

            return string.Format("UPDATE {0} SET {1} WHERE {2}={3}{2}",
                  t.TableInfo.TableName,
                  string.Join(",", cols),
                  t.TableInfo.PrimaryKey,
                  _paramPrefix);
        }
        #endregion

        #region SQL条件拼接
        private readonly Regex reg = new Regex(@"^(?<tab>\w+\.)?(?<Pre>(PK|Begin|End|Like|UnLike|Null|In))?(?<Key>\w+)$");

        private string CreateWhere(PocoData t, IDictionary<string, object> condition)
        {
            var sql = new StringBuilder();

            //遍历条件
            foreach (var key in condition.Keys.ToArray())
            {
                var match = reg.Match(key); //正则获取到属性对应的字段名
                var pre = match.Groups["Pre"].Value;//获取命名规则中的比较谓词
                var colName = match.Groups["Key"].Value;//获取字段名称
                //if (string.IsNullOrEmpty(tableName))
                //    tableName = match.Groups["tab"].Value;//获取字段表名限定

                var paraName = string.Format("{0}{1}", pre, colName);//获取该条件被分配的过程参数名

                var value = condition[key];

                //拼接Sql语句
                sql.AppendFormat(" {0} AND",
                    Builder(pre, colName, paraName, t.TableInfo.TableName, value));//拼接SQL条件

                if (pre == "In") //In拼接没有使用过程参数，直接拼接了，将参数从字典中移除
                {
                    condition.Remove(key);
                    continue;
                }

                //若该字段值的类型为字符串，检测实体中的类型
                if (value.GetType() == typeof(string))
                {
                    PocoColumn prop;
                    if (!t.Columns.TryGetValue(colName, out prop))
                        throw new Exception(string.Format("can't find any property in object.propertyName:{0},object:{1}", key, t.TableInfo.TableName));

                    if (prop.PropertyInfo.PropertyType != typeof(string))
                        condition[key] = prop.ChangeType(value);
                }
            }

            if (sql.Length > 3) sql.Length -= 3;
            return sql.ToString();
        }

        private string Builder(string pre, string colName, string paraName, string table, object colValue = null)
        {
            table += ".";
            switch (pre)
            {
                case "Like": return Like(colName, paraName, table);
                case "UnLike": return Like(colName, paraName, table, false);
                case "Begin": return Begin(colName, paraName, table);
                case "End": return End(colName, paraName, table);
                case "Equal": return Equal(colName, paraName, table);
                case "In": return In(colName, paraName, table, colValue);
                case "Null":
                    bool value = false;
                    if (colValue != null && bool.TryParse(colValue.ToString(), out value) && value) ;
                    return Null(colName, table, value);
                default: return Equal(colName, paraName, table);
            }
        }
        protected virtual string Equal(string colName, string paraName, string tab)
        {
            return string.Format("{0}{1} = {2}{3}", tab, colName, _paramPrefix, paraName);
        }
        protected virtual string Like(string colName, string paraName, string tab, bool not = true)
        {
            var cSymbol = _paramPrefix == '@' ? "+" : "||";
            return string.Format("{0}{1} {2}like '%'{3}{4}{5}{3}'%'", tab, colName, not ? string.Empty : "not ", cSymbol, _paramPrefix, paraName);
        }
        protected virtual string Begin(string colName, string paraName, string tab)
        {
            return string.Format("{0}{1} >= {2}{3}", tab, colName, _paramPrefix, paraName);
        }
        protected virtual string End(string colName, string paraName, string tab)
        {
            return string.Format("{0}{1} <= {2}{3}", tab, colName, _paramPrefix, paraName);
        }
        protected virtual string Null(string colName, string tab, bool isNull)
        {
            return string.Format("{0}{1} is {2}null", tab, colName, isNull ? string.Empty : "not ");
        }
        protected virtual string In(string colName, string paraName, string tab, object colValue)
        {
            return string.Format("{0}{1} in ({2})", tab, colName, colValue);
        }
        #endregion

        #region 生成 分页SQL语句
        /// <summary>
        /// 匹配移除Select后的sql
        /// </summary>
        private Regex rxColumns = new Regex(@"\A\s*SELECT\s+((?:\((?>\((?<depth>)|\)(?<-depth>)|.?)*(?(depth)(?!))\)|.)*?)(?<!,\s+)\bFROM\b", RegexOptions.IgnoreCase | RegexOptions.Multiline | RegexOptions.Singleline | RegexOptions.Compiled);
        /// <summary>
        /// 匹配SQL语句中Order By字段
        /// </summary>
        private Regex rxOrderBy = new Regex(@"\b(?<ordersql>ORDER\s+BY\s+(?:\((?>\((?<depth>)|\)(?<-depth>)|.?)*(?(depth)(?!))\)|[\w\(\)\.])+)(?:\s+(?<order>ASC|DESC))?(?:\s*,\s*(?:\((?>\((?<depth>)|\)(?<-depth>)|.?)*(?(depth)(?!))\)|[\w\(\)\.])+(?:\s+(?:ASC|DESC))?)*", RegexOptions.IgnoreCase | RegexOptions.Multiline | RegexOptions.Singleline | RegexOptions.Compiled);
        /// <summary>
        /// 匹配SQL语句中Distinct
        /// </summary>
        private Regex rxDistinct = new Regex(@"\ADISTINCT\s", RegexOptions.IgnoreCase | RegexOptions.Multiline | RegexOptions.Singleline | RegexOptions.Compiled);
        /// <summary>
        /// 分析Sql语句 输出分析数组 信息依次为:
        /// 0.countsql
        /// 1.pageSql(保留位置此处不做分析)
        /// 2.移除了select的sql
        /// 3.order by 字段 desc
        /// 4.order by 字段
        /// 5.desc
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        private string[] SplitSqlForPaging(string sql)
        {
            var sqlInfo = new string[6];
            // Extract the columns from "SELECT <whatever> FROM"
            var m = rxColumns.Match(sql);
            if (!m.Success)
                return null;

            // Save column list and replace with COUNT(*)
            Group g = m.Groups[1];
            sqlInfo[2] = sql.Substring(g.Index);

            if (rxDistinct.IsMatch(sqlInfo[2]))
                sqlInfo[0] = sql.Substring(0, g.Index) + "COUNT(" + m.Groups[1].ToString().Trim() + ")  TOTAL " + sql.Substring(g.Index + g.Length);
            else
                sqlInfo[0] = sql.Substring(0, g.Index) + "COUNT(*) TOTAL " + sql.Substring(g.Index + g.Length);


            // Look for an "ORDER BY <whatever>" clause
            m = rxOrderBy.Match(sqlInfo[0]);
            if (!m.Success)
            {
                sqlInfo[3] = null;
            }
            else
            {
                g = m.Groups[0];
                sqlInfo[3] = g.ToString();
                //统计的SQL 移除order
                sqlInfo[0] = sqlInfo[0].Substring(0, g.Index) + sqlInfo[0].Substring(g.Index + g.Length);
                //存储排序信息
                sqlInfo[4] = m.Groups["ordersql"].Value;//order by xxx
                sqlInfo[5] = m.Groups["order"].Value;//desc 

                //select部分 移除order
                sqlInfo[2] = sqlInfo[2].Replace(sqlInfo[3], string.Empty);
            }

            return sqlInfo;
        }

        /// <summary>
        /// 生成逆序分页Sql语句
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="sqls"></param>
        /// <param name="start"></param>
        /// <param name="limit"></param>
        /// <param name="total"></param>
        private bool CreatePageSqlReverse(string sql, ref string[] sqls, int start, int limit, int total = 0)
        {
            //如果总行数不多或分页的条数位于前半部分，没必要逆序分页
            if (total < 200000 || start <= total / 2)
            {
                return false;
            }

            //sql正则分析过后的数组有5个值，若未分析，此处分析
            if (sqls == null || sqls.Length == 6)
            {
                sqls = SplitSqlForPaging(sql);
                if (sqls == null)
                {
                    //无法解析的SQL语句
                    throw new Exception("can't parse sql to pagesql ,the sql is " + sql);
                }
            }

            //如果未定义排序规则，则无需做逆序分页计算
            if (string.IsNullOrEmpty(sqls[5]))
            {
                return false;
            }

            //逆序分页检查
            string sqlOrder = sqls[3];
            int end = start + limit;

            //获取逆序排序的sql
            string sqlOrderChange = string.Compare(sqls[5], "desc", true) == 0 ?
                string.Format("{0} ASC ", sqls[4]) :
                string.Format("{0} DESC ", sqls[4]);

            /*理论
             * total:10000 start:9980 limit:10 
             * 则 end:9990 分页条件为 RN >= 9980+1 and RN <= 9990
             * 逆序调整后 
             * start = total - start = 20
             * end = total - end + 1 = 11
             * 交换start和end，分页条件为 RN >= 11 and RN<= 20
             */
            //重新计算start和end
            start = total - start;
            end = total - end + 1;
            //交换start end
            start = start + end;
            end = start - end;
            start = start - end;


            //定义分页SQL
            var pageSql = new StringBuilder();

            if (_dbType == DBType.SqlServer2000)
            {
                pageSql.AppendFormat("SELECT TOP @PageLimit * FROM ( SELECT TOP @PageEnd {0} {1} ) ", sqls[2], sqlOrderChange);
            }
            else if (_dbType == DBType.SqlServer)
            {
                //组织分页SQL语句
                pageSql.AppendFormat("SELECT PageTab.* FROM ( SELECT TOP (@PageEnd) ROW_NUMBER() over ({0}) RN , {1}  ) PageTab ",
                    sqlOrderChange,
                    sqls[2]);

                //如果查询不是第一页，则需要判断起始行号
                if (start > 1)
                {
                    pageSql.Append("Where RN >= :PageStart ");
                }
            }
            else if (_dbType == DBType.Oracle)
            {
                pageSql.AppendFormat("SELECT ROWNUM RN,  PageTab.* FROM  ( Select {0} {1} ) PageTab  where ROWNUM <= :PageEnd ", sqls[2], sqlOrderChange);

                //如果查询不是第一页，则需要判断起始行号
                if (start > 1)
                {
                    pageSql.Insert(0, "SELECT * FROM ( ");
                    pageSql.Append(" ) ");
                    pageSql.Append(" WHERE RN>= :PageStart ");
                }
            }
            else if (_dbType == DBType.SQLite)
            {
                pageSql.AppendFormat("SELECT * FROM ( SELECT {0} {1} limit  @PageStart,@PageLimit ) PageTab ", sqls[2], sqlOrderChange);
            }

            //恢复排序
            pageSql.Append(sqlOrder);

            //存储生成的分页SQL语句  
            sqls[1] = pageSql.ToString();

            //临时测试
            //sqls[1] = sqls[1].Replace("@", "").Replace(":", "").Replace("PageStart", start + "").Replace("PageEnd", end + "").Replace("PageLimit", limit + "");
            return true;
        }

        /// <summary>
        /// 生成常规Sql语句
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="start"></param>
        /// <param name="limit"></param>
        /// <param name="createCount"></param>
        private string[] CreatePageSql(string sql, int start, int limit, bool createCount = false)
        {
            //需要输出的sql数组
            string[] sqls = null;

            //生成count的SQL语句 SqlServer生成分页，必须通过正则拆分
            if (createCount || _dbType == DBType.SqlServer || _dbType == DBType.SqlServer2000)
            {
                sqls = SplitSqlForPaging(sql);
                if (sqls == null)
                {
                    //无法解析的SQL语句
                    throw new Exception("can't parse sql to pagesql ,the sql is " + sql);
                }
            }
            else
            {
                sqls = new string[2];
            }

            //组织分页SQL语句
            var pageSql = new StringBuilder();

            //构建分页参数
            var end = start + limit;
            start++;

            if (_dbType == DBType.SqlServer2000)
            {
                pageSql.AppendFormat("SELECT TOP @PageEnd {0} {1}", sqls[2], sqls[3]);

                if (start > 1)
                {
                    var orderChange = string.IsNullOrEmpty(sqls[5]) ? null :
                        string.Compare(sqls[5], "desc", true) == 0 ?
                        string.Format("{0} ASC ", sqls[4]) :
                        string.Format("{0} DESC ", sqls[4]);
                    pageSql.Insert(0, "SELECT TOP 100 PERCENT  * FROM (SELECT TOP @PageLimit * FROM ( ");
                    pageSql.AppendFormat(" ) PageTab {0} ) PageTab2 {1}", orderChange, sqls[3]);
                }
            }
            else if (_dbType == DBType.SqlServer)
            {
                pageSql.AppendFormat("SELECT TOP (@PageEnd) ROW_NUMBER() OVER ({0}) RN , {1}",
                    string.IsNullOrEmpty(sqls[3]) ? "ORDER BY (SELECT NULL)" : sqls[3],
                    sqls[2]);

                //如果查询不是第一页，则需要判断起始行号
                if (start > 1)
                {
                    pageSql.Insert(0, "SELECT PageTab.* FROM ( ");
                    pageSql.Append(" ) PageTab WHERE RN >= @PageStart");
                }
            }
            else if (_dbType == DBType.Oracle)
            {
                pageSql.Append("SELECT ROWNUM RN,  PageTab.* FROM ");
                pageSql.AppendFormat(" ( {0} ) PageTab ", sql);
                pageSql.Append(" WHERE ROWNUM <= :PageEnd ");

                //如果查询不是第一页，则需要判断起始行号
                if (start > 1)
                {
                    pageSql.Insert(0, "SELECT * FROM ( ");
                    pageSql.Append(" ) WHERE RN>= :PageStart ");
                }
            }
            else if (_dbType == DBType.SQLite)
            {
                pageSql.AppendFormat("{0} LIMIT @PageStart,@PageLimit", sql, start, limit);
            }

            //存储生成的分页SQL语句  
            sqls[1] = pageSql.ToString();

            //临时测试
            //sqls[1] = sqls[1].Replace("@", "").Replace(":", "").Replace("PageStart", start + "").Replace("PageEnd", end + "").Replace("PageLimit", limit + "");
            return sqls;
        }

        private void CreatePageParameter(IDictionary<string, object> condition, int start, int limit, DbCommand cmd = null)
        {
            var paraName = string.Empty;
            int paraValue = -1;
            //组织过程参数
            if (_dbType == DBType.SqlServer2000 || _dbType == DBType.SQLite)
            {
                paraName = _paramPrefix + "PageLimit";
                paraValue = start + 1;

                if (condition != null) condition[paraName] = paraValue;
                else if (cmd != null)
                {
                    var para = cmd.CreateParameter();
                    para.ParameterName = paraName;
                    para.Value = paraValue;
                    cmd.Parameters.Add(para);
                }
            }
            else if (start > 1)
            {
                paraName = _paramPrefix + "PageStart";
                paraValue = start + 1;

                if (condition != null) condition[paraName] = paraValue;
                else if (cmd != null)
                {
                    var para = cmd.CreateParameter();
                    para.ParameterName = paraName;
                    para.Value = paraValue;
                    cmd.Parameters.Add(para);
                }
            }

            paraName = _paramPrefix + "PageEnd";
            paraValue = start + limit;

            if (condition != null) condition[paraName] = start + limit;
            if (cmd != null)
            {
                var para = cmd.CreateParameter();
                para.ParameterName = paraName;
                para.Value = paraValue;
                cmd.Parameters.Add(para);
            }
        }
        #endregion

    }
}
